
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Utils";

TobiiWindow = class( Turbine.UI.Lotro.Window );

function TobiiWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	-- Position saving flag
	local hasMoved = false;

	-- Initialize the container that holds the elements referencing the player's inventory.
	self.items = { };
	self.backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();

	self:AdjustSize( self.backpack:GetSize() );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetText( "Tobii" );
	self:SetOpacity( 1 );
	self:SetWantsKeyEvents( true );

	-- Loads saved coordinates, or use default coords, and save them
	self:Load();

	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetAllowDrop( true );

	-- event handlers
	self.PositionChanged = function( sender, args )
		hasMoved = true;
	end

	self.VisibleChanged = function( sender, args )
		if( ( not sender:IsVisible() ) and hasMoved ) then
			self:Save();
			hasMoved = false;
		end
	end

	self.Closing = function( sender, args )
		if ( ( not sender:IsVisible() ) and hasMoved ) then
			self:Save();
			hasMoved = false;
		end
	end

	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( false ) 
		end

		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) 
		end
	end

	self.itemListBox.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();
		if ( shortcut ~= nil ) then
			local destinationItemControl = self.itemListBox:GetItemAt( args.X, args.Y );

			-- if nil, then it's out of range, so don't do anything
			if ( destinationItemControl ~= nil ) then
				local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );

				self.backpack:PerformShortcutDrop(
					shortcut,
					destinationIndex,
					Turbine.UI.Control.IsShiftKeyDown() );
			end
		end
	end

	self.backpack.SizeChanged = function( sender, args )
		self:AdjustSize( self.backpack:GetSize() );
		self:Nudge();
		self:Refresh();
	end

	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
	end

	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
	end

	self.backpack.ItemMoved = function( sender, args )
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end

	self:Refresh();
end

function TobiiWindow:Load()
	local location = Turbine.PluginData.Load( Turbine.DataScope.Character, "TobiiLocation" );

	-- if any checks are true, data is bad/invalid
	if ( ( type( location ) ~= "table" )
		or ( type( location.x ) ~= "number")
		or ( location.x >= Turbine.UI.Display:GetWidth() )
		or ( type( location.y ) ~= "number")
		or ( location.y >= Turbine.UI.Display:GetHeight() ) )
	then
		-- assign a default location
		local edgePadding = 50;

		self:SetPosition(
			Turbine.UI.Display.GetWidth() - self:GetWidth() - edgePadding,
			Turbine.UI.Display.GetHeight() - self:GetHeight() - edgePadding * 1.5 );

		self:Save();
	else
		-- valid coordinates and data structure, use them
		self:SetPosition(location.x, location.y);

		-- perform nudge in case user gained bag space between last save and this load
		self:Nudge();
	end
end

function TobiiWindow:Save()
	local location = { };
	location.x, location.y = self:GetPosition();
	Turbine.PluginData.Save( Turbine.DataScope.Character, "TobiiLocation", location );
end

function TobiiWindow:Refresh()
	-- dumps content of list box, prevents the items being displayed multiple times
	if(self.itemListBox:GetItemCount() ~= 0) then
		self.itemListBox:ClearItems();
	end
	
	local backpackSize = self.backpack:GetSize();
	local temp;

	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end
		
		-- Get item and set background color
		temp = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		temp:SetBackground( "Almagnus1/Tobii/Resources/slotBackground.tga" );
		temp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		
		self.items[i] = temp;
		self.itemListBox:AddItem( self.items[i] );
	end

	self:PerformLayout();
end

function TobiiWindow:AdjustSize( backPackSize)
	if (backPackSize == 75) then self:SetSize( 422, 303 );
	elseif (backPackSize == 45) then self:SetSize( 352, 233 );
	elseif (backPackSize == 60) then self:SetSize( 386, 268 );
	end
end

-- Adjusts Tobii's position, so that part of it isn't hanging off the screen.
function TobiiWindow:Nudge()
	local width = self:GetWidth();
	local height = self:GetHeight();
	local screenWidth = Turbine.UI.Display:GetWidth();
	local screenHeight = Turbine.UI.Display:GetHeight();

	-- check horizontal
	if (self:GetLeft() + width > screenWidth + 1) then
		self:SetLeft( screenWidth - width );
	end

	-- check vertical
	if (self:GetTop() + height > screenHeight + 1) then
		self:SetTop( screenHeight - height );
	end
end

function TobiiWindow:PerformLayout()
	self:Layout( { } )
end

function TobiiWindow:Layout( args )
	local width, height = self:GetSize();
	
	local itemWidth = 40;
	
	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = width - 20;
	local listHeight = height - 50;
	local itemsPerRow = listWidth / itemWidth;
	
	self.itemListBox:SetPosition( 15, 35 );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
end